import { INetResponse } from "@/Engine/CatanEngine/NetManagerV2/Core/INetResponse";
import CSSettingsV3 from "@/FormTable/CSSettingsV3";
import { confirmModalObj } from "@/UI/MessageBox/ConfirmModalContext";
import { modalObj } from "@/UIControl/ModalContext";
import { Cocos } from "@/assets/VueScript/Cocos";
import { CommonEventCallBack, JackpotPoolCallBack } from "@/assets/VueScript/CocosVueScript";
import GameData_Cocos from "@/assets/VueScript/share/GameData_Cocos";
import { chatLog, chatMessage, setChatBanTime } from "@/components/ModalContent/ChatRoomModal/chatUtils";
import Config from "@/define/Config";
import Player from "@/modules/player";
import UserData from "@/modules/player/UserData";
import { State } from "@/modules/player/define/data";
import { ILineShareData, ModalContentType } from "@/types";
import { CommonEventType } from "@/utils";
import { NumberEx } from "@/utils/Number/NumberEx";
import { activityComSync, backpackInfo, friendAllowList, friendDenyList, profileInfo, txnCenter, txnNew, txnTrade, txnUserAdd, vipInfo } from "@/utils/setRPCData";
import MainControl from "../MainControl/MainControl";
import CSMessage from "../Message/CSMessage";
import CSResource from "../ResourceItem/CSResource";
import IResourceItem from "../ResourceItem/IResourceItem";

export default class MainControlData {

	/**
	Normal = 0,         //一般斷線or登出
	RpcError = 1,       //RPC錯誤
	SystemError = 2,    //系統錯誤
	UserError = 3,      //使用者造成的錯誤
	DbaError = 4,       //query時回傳錯誤
	SendBufferError = 5, //儲存傳送資料的buff錯誤
	Kick = 6,            //指定踢除
	ConnectLimit = 7,    //連線限制
	RequireHardLimit = 8,   //機台請求逾時
	SignalWallet = 9,
	RepeatLogin = 101, //重覆登入
	LoginFailed = 102, //登入失敗
	AppVerError = 103, //Client版本過舊
	Maintain = 104,    //維護中
	Ban = 105,         //帳號鎖定
	SlotClose = 106,   //機台關閉
	ResourceWait = 107, //使用者資料尚未寫入
	IsDelete = 108, // 被刪除的帳號
	Delete = 109, // 刪除主動斷線
	SafeDisconnect = 201,    // 安全離線
	*/
	private _disconnectErrorType: number = null;

	constructor() {
		Player.DataReceivedEvent.AddCallback(this._dataReceivedEvent, this);
	}
	private _dataReceivedEvent(param: any[] = null): void {
		let type: MainControl.DataType = param[0];
		let data: any = param[1];
		switch (type) {
			case MainControl.DataType.ServerData:
				this._serverData(data);
				break;
			case MainControl.DataType.NetDisconnected:
				this._netDisconnected();
				break;
			default:
				break;
		}
	}
	// =======================================================================================
	/** SERVER主動通知 */
	private _serverData(resp: INetResponse<any>): void {
		if (resp.IsValid) {
			switch (resp.Method) {
				case "activity_com.sync": {
					activityComSync(resp.Data);
					break;
				}
				case "backpack.info": {
					backpackInfo(resp.Data);
					break;
				}
				case "chat.ban": {
					setChatBanTime(resp.Data);
					break;
				}
				case "chat.message": {
					chatMessage(resp.Data);
					break;
				}
				case "chat.log": {
					chatLog(resp.Data);
					break;
				}
				case "friend.allow_list": {
					friendAllowList(resp.Data);
					break;
				}
				case "friend.deny_list": {
					friendDenyList(resp.Data);
					break;
				}
				case "game.share": {
					const { openLineShareGame } = confirmModalObj;
					const data: ILineShareData = {
						slotID: resp.Data[0],
						winMilt: resp.Data[1]
					};
					openLineShareGame(data);
					break;
				}
				case "game.sync": {
					Cocos.VicKing_Bridge.InGameGetUUID = true;
					break;
				}
				case "jackpot.get": {
					const { handleOpen, setModalType } = modalObj;
					const playerData: State = Player.data.getState();
					playerData.account.jackpotGet.push(...resp.Data);
					Player.data.setState(playerData);
					setModalType(ModalContentType.jackpot);
					handleOpen();
					break;
				}
				case "jackpot.pool": {
					const playerData: State = Player.data.getState();
					if (!Config.IsVite || MainControl.Instance.IsInGame) {
						for (let i = 0; i < playerData.game.jackpotPool.length; i++) {
							const oldJpData: [gameId: number, maxJPId: number, jp: number] = playerData.game.jackpotPool[i];
							let newJpData: [gameId: number, maxJPId: number, jp: number] = null;
							for (let j = 0; j < resp.Data.length; j++) {
								const [gameId, maxJPId, jp] = resp.Data[j];
								if (gameId === oldJpData[0]) {
									newJpData = resp.Data[j];
									break;
								}
							}
							if (!newJpData) {
								JackpotPoolCallBack.DispatchCallback(oldJpData[0], 0);
							}
						}
					}
					playerData.game.jackpotPool = resp.Data;
					Player.data.setState(playerData);
					if (!Config.IsVite || MainControl.Instance.IsInGame) {
						for (let i = 0; i < playerData.game.jackpotPool.length; i++) {
							const jpData: [gameId: number, maxJPId: number, jp: number] = playerData.game.jackpotPool[i];
							JackpotPoolCallBack.DispatchCallback(jpData[0], jpData[2]);
						}
					} else {
						Cocos.CocosEventListener.DispatchCallback(GameData_Cocos.CELT.UpdateJPPool, null);
					}
					break;
				}
				case "maintain.info": {
					const playerData: State = Player.data.getState();
					if (resp.Data) {
						resp.Data[3] = NumberEx.plus(MainControl.Instance.NowTime, resp.Data[3]);
						playerData.maintain = resp.Data;
						CommonEventCallBack.DispatchCallback(CommonEventType.Maintenance, null);
					} else {
						playerData.maintain = undefined;
					}
					Player.data.setState(playerData);
					break;
				}
				case "profile.info": {
					profileInfo(resp.Data);
					break;
				}
				case "resource.update": {
					const resourceItems: IResourceItem[] = CSResource.GetResourceItemsFromServer(resp.Data);
					UserData.DoResUpdate(resourceItems);
					break;
				}
				case "resource.bankruptcy": {
					Cocos.CocosEventListener.DispatchCallback(GameData_Cocos.CELT.Bankruptcy, null);
					break;
				}
				case "system.disconnect": {
					this._disconnectErrorType = +resp.Data["c"];
					break;
				}
				case "txn.new": {
					txnNew(resp.Data);
					break;
				}
				case "txn.center": {
					txnCenter(resp.Data);
					break;
				}
				case "txn.trade": {
					txnTrade(resp.Data);
					break;
				}
				case "txn.user_add": {
					txnUserAdd(resp.Data);
					break;
				}
				case "vip.info": {
					vipInfo(resp.Data);
					break;
				}
				case "vip.level": {
					const playerData: State = Player.data.getState();
					playerData.vip.level = resp.Data;
					Player.data.setState(playerData);
					break;
				}
				default:
					break;
			}
		} else {
			switch (resp.Method) {
				case "chat.log": {
					chatLog([null, null]);
					break;
				}
				default:
					break;
			}
		}
	}
	// =======================================================================================
	/** SOCKET斷線 */
	private _netDisconnected(): void {
		console.debug("Disconnected Error Type : " + this._disconnectErrorType);
		let str: string = null;
		if (this._disconnectErrorType < 10 && this._disconnectErrorType >= 0) {
			str = CSSettingsV3.prototype.CommonString(55 + this._disconnectErrorType);
		} else if (this._disconnectErrorType > 100 && this._disconnectErrorType < 110) {
			str = CSSettingsV3.prototype.CommonString(65 + this._disconnectErrorType - 101);
		} else {
			str = "Server Disconnected";
		}
		CSMessage.CreateYesMsg(
			str,
			this._disconnectedReload
		);
	}

	private _disconnectedReload(): void {
		window.location.reload();
	}

	// =======================================================================================
}