import { BusinessEnum } from "@/_BusinessTypeSetting/BusinessTypeSetting";
import { useGameItems } from "@/context/GameItemsContext";
import { Button, Cascader } from "antd";
import React, { useState } from "react";

interface Option {
	value: string;
	label: string;
	children?: Option[];
}

const Login = () => {
	const { onLoad } = useGameItems();
	const serverType: typeof BusinessEnum.ServerType = BusinessEnum.ServerType;
	const [type, setType] = useState<number>(BusinessEnum.ServerType.Internal_Dev);
	const [isLogin, setIsLogin] = useState<boolean>(false);
	const options: Option[] = [];
	for (let i = 0, names: string[] = Object.keys(serverType); i < names.length; i++) {
		const key: string = names[i];
		if (!Number.isNaN(+key)) {
			options.push({
				value: key,
				label: serverType[key],
			});
		}
	}

	async function login() {
		setIsLogin(true);
		await onLoad(type);
	}

	return (
		<>{!isLogin &&
			<div style={boxStyle}>
				<Cascader defaultValue={[BusinessEnum.ServerType[BusinessEnum.ServerType.Internal_Dev]]} options={options} onChange={(v: string[]) => setType(+v[0])} />
				<Button type="primary" onClick={login}>登入</Button>
			</div>
		}</>
	);
};

export default Login;

const boxStyle: React.CSSProperties = {
	width: "100%",
	height: "100vh",
	borderRadius: 6,
	border: "1px solid #40a9ff",
	display: "flex",
	alignItems: "center",
	justifyContent: "center",
};